/*
# PostgreSQL Database Modeler (pgModeler)
#
# (c) Copyright 2006-2026 - Raphael Araújo e Silva <raphael@pgmodeler.io>
#
# DEVELOPMENT, MAINTENANCE AND COMMERCIAL DISTRIBUTION BY:
# Nullptr Labs Software e Tecnologia LTDA <contact@nullptrlabs.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 3.
#
# This program is distributed in the hope that it will be useful),
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The complete text of GPLv3 is at LICENSE file on source code root directory.
# Also), you can get the complete GNU General Public License at <http://www.gnu.org/licenses/>
*/

#include "attributes.h"

namespace Attributes {
	const QString
	Accent("accent"),
	Action("action"),
	Active("active"),
	ActiveLayers("active-layers"),
	AddedItem("added-item"),
	AdminOption("admin-option"),
	AdminRoles("admin-roles"),
	AlertApplyMetadata("alert-apply-metadata"),
	AlertDisableQuoting("alert-disable-quoting"),
	AlertOpenSqlTabs("alert-open-sqltabs"),
	AlertUnsavedModels("alert-unsaved-models"),
	Alias("alias"),
	Alignment("alignment"),
	AlignObjsToGrid("align-objs-to-grid"),
	AllowCompletion("allow-completion"),
	AllowConns("allow-conns"),
	AllVersions("all"),
	AlternateBase("alternate-base"),
	AlterCmds("alter-cmds"),
	AnalyzeFunc("analyze"),
	AncestorTable("ancestor-table"),
	Any("any"),
	AppendAtEod("append-at-eod"),
	AppendedSql("appended-sql"),
	Application("application"),
	ApplyOnServer("apply-on-server"),
	ArgCount("arg-count"),
	ArgDefCount("arg-def-count"),
	ArgDefaults("arg-defaults"),
	ArgModes("arg-modes"),
	ArgNames("arg-names"),
	ArgTypes("arg-types"),
	Arguments("arguments"),
	AscOrder("asc-order"),
	Assignment("assignment"),
	Attribute("attribute"),
	Attributes("attributes"),
	AttribsPage("attribs-page"),
	AttribsPerPage("attribs-per-page"),
	AutoBrowseDb("auto-browse-db"),
	AutoSaveInterval("autosave-interval"),
	Author("author"),
	BgColor("bg-color"),
	Base("base"),
	BaseType("base"),
	Before("before"),
	BehaviorType("behavior-type"),
	Big("big"),
	BlogPost("blogpost"),
	Bold("bold"),
	BorderColor("border-color"),
	Bottom("bottom"),
	BrightText("bright-text"),
	Buffering("buffering"),
	Button("button"),
	ButtonText("button-text"),
	BypassRls("bypassrls"),
	ByValue("by-value"),
	Cache("cache"),
	CanonicalFunc("canonical"),
	CanvasColor("canvas-color"),
	CanvasCornerMove("canvas-corner-move"),
	CaptureNearby("capture-nearby"),
	Cascade("cascade"),
	CaseSensitive("case-sensitive"),
	CastType("cast-type"),
	Category("category"),
	Change("change"),
	Changelog("changelog"),
	CheckExp("check-exp"),
	CheckOption("check-option"),
	CheckUpdate("check-update"),
	CheckVersions("check-versions"),
	CkConstr("ck-constr"),
	ClientEncoding("client-encoding"),
	CloseChar("close-char"),
	Code("code"),
	CodeCompletion("code-completion"),
	ColIndexes("col-indexes"),
	ColIsIdentity("col-is-identity"),
	CollapseMode("collapse-mode"),
	Collatable("collatable"),
	Collation("collation"),
	Collations("collations"),
	Color("color"),
	Colors("colors"),
	ColsComment("cols-comment"),
	Column("column"),
	ColumnAlias("column-alias"),
	ColumnTruncThreshold("column-trunc-threshold"),
	Columns("columns"),
	Command("command"),
	Commands("commands"),
	Comment("comment"),
	CommutatorOp("commutator-op"),
	CompactView("compact-view"),
	CompareToDatabase("compare-to-db"),
	ComparisonType("comparison-type"),
	CompletionTrigger("completion-trigger"),
	CompositeType("composite"),
	Concurrent("concurrent"),
	Condition("condition"),
	ConfigFile("config-file"),
	ConfigParam("confparam"),
	ConfigParams("config-params"),
	Configuration("configuration"),
	ConfirmValidation("confirm-validation"),
	ConnectCenterPnts("center-pnts"),
	ConnectFkToPk("fk-to-pk"),
	Connection("connection"),
	ConnectionTimeout("connection-timeout"),
	Connections("connections"),
	ConnectPriv("connect"),
	ConnectTableEdges("table-edges"),
	ConnLimit("connlimit"),
	Constraint("constraint"),
	Constraints("constraints"),
	ConstrDefs("constr-defs"),
	ConstrIndexes("constr-indexes"),
	ConstrSqlDisabled("constr-sql-disabled"),
	Contents("contents"),
	CopyMode("copy-mode"),
	CopyOptions("copy-options"),
	CopyTable("copy-table"),
	Create("create"),
	CreateCmds("create-cmds"),
	CreateDb("createdb"),
	CreatePriv("create"),
	CreateRole("createrole"),
	Created("created"),
	CrowsFoot("crows-foot"),
	CteExpression("cte-exp"),
	CurIdentityType("cur-identity-type"),
	CurVersion("cur-version"),
	CurrentModel("current-model"),
	CustomColor("custom-color"),
	CustomFilter("custom-filter"),
	CustomIdxs("customidxs"),
	CustomScale("custom-scale"),
	Cycle("cycle"),
	Dark("dark"),
	DataDirectory("data-directory"),
	DataDictIndex("datadictidx"),
	Database("database"),
	Date("date"),
	DbModel("dbmodel"),
	DdlEndToken("-- ddl-end --"),
	DeadRowsAmount("dead-rows-amount"),
	DebugMode("debug-mode"),
	Declaration("declaration"),
	DeclInTable("decl-in-table"),
	Default("default"),
	DefaultCollation("default-collation"),
	DefaultForOps("default-for-ops"),
	DefaultOwner("default-owner"),
	DefaultSchema("default-schema"),
	DefaultTablespace("default-tablespace"),
	DefaultValue("default-value"),
	Deferrable("deferrable"),
	DeferType("defer-type"),
	Definition("definition"),
	DelAction("del-action"),
	DelEvent("del-event"),
	Deleted("deleted"),
	DeletePriv("delete"),
	Delimiter("delimiter"),
	DelimitersColor("delimiters-color"),
	Description("description"),
	Design("design"),
	DestType("destiny-type"),
	Deterministic("deterministic"),
	Diff("diff"),
	DiffDatabases("diff-databases"),
	DiffModelDb("diff-model-db"),
	DiffModels("diff-models"),
	DifMode("diff-mode"),
	Dimension("dimension"),
	Directory("directory"),
	Disabled("disabled"),
	DisableInlineEditor("disable-inline-editor"),
	DisableNameQuoting("disable-name-quoting"),
	DisableSmoothness("disable-smoothness"),
	DisplayLineNumbers("display-line-numbers"),
	DockWidgets("dock-widgets"),
	DomConstraint("domconstraint"),
	DontDropMissingObjs("dont-drop-missing-objs"),
	Dot("dot"),
	Drop("drop"),
	DropCmds("drop-cmds"),
	DropMissingColsConstrs("drop-missing-cols-constrs"),
	DropTruncCascade("drop-trunc-cascade"),
	DstColPattern("dst-col-pattern"),
	DstColumns("dst-columns"),
	DstEncoding("dst-encoding"),
	DstFkPattern("dst-fk-pattern"),
	DstLabel("dst-label"),
	DstRequired("dst-required"),
	DstTable("dst-table"),
	DstType("dst-type"),
	DynamicLibraryPath("dynamic-library-path"),
	DynamicSharedMemory("dynamic-shared-memory-type"),
	Element("element"),
	Elements("elements"),
	ElementsCount("elements-count"),
	EmptyPassword("empty-password"),
	Encoding("encoding"),
	EnclosingChars("enclosing-chars"),
	Encrypted("encrypted"),
	EndExp("end-exp"),
	EntireLine("entire-line"),
	Entry("entry"),
	EnumType("enumeration"),
	EscapeComment("escape-comment"),
	Event("event"),
	EventType("event-type"),
	Events("events"),
	ExactMatch("exact-match"),
	ExcBuiltinArrays("exc-builtin-arrays"),
	ExcludeElement("excelement"),
	ExConstr("ex-constr"),
	ExecType("exec-type"),
	ExecutPriv("execute"),
	ExecutionCost("execution-cost"),
	ExistingValue("existing-value"),
	ExpansionFactor("expansion-factor"),
	Explicit("explicit"),
	Export("export"),
	ExportToFile("export-to-file"),
	Expression("expression"),
	Expressions("expressions"),
	ExtAttribsPage("ext-attribs-page"),
	ExtAttribsPerPage("ext-attribs-per-page"),
	ExtObjOids("ext-obj-oids"),
	ExtraCondition("extra-condition"),
	ExtraInfo("extra-info"),
	Factor("factor"),
	FadedOut("faded-out"),
	FadeInObjects("fadein-objects"),
	False("false"),
	Family("family"),
	FastUpdate("fast-update"),
	Fdw("fdw"),
	File("file"),
	FileAssociated("file-associated"),
	FillColor("fill-color"),
	Filter("filter"),
	FilterOids("filter-oids"),
	FilterTableTypes("filter-tab-types"),
	Filters("filters"),
	Final("final"),
	FinalExp("final-exp"),
	FinalFunc("final"),
	FiringType("firing-type"),
	FirstRun("first-run"),
	FkColumn("fk-column"),
	FkConstr("fk-constr"),
	FkDefs("fk-defs"),
	FkIdxPattern("fk-idx-pattern"),
	FkIdxType("fk-idx-type"),
	Font("font"),
	FontSize("font-size"),
	ForceObjsReCreation("force-objs-re-creation"),
	ForcedFiltering("forced-filtering"),
	ForeignKeys("foreign-keys"),
	ForeignTableBody("foreigntable-body"),
	ForeignTableExtBody("foreigntable-ext-body"),
	ForeignTableName("foreigntable-name"),
	ForeignTableSchemaName("foreigntable-schema-name"),
	ForeignTableTitle("foreigntable-title"),
	ForeignTableTogglerBody("foreigntable-toggler-body"),
	ForeignTableTogglerButtons("foreigntable-toggler-btns"),
	FgColor("fg-color"),
	FormatName("format-name"),
	FromExp("from-exp"),
	FromSqlFunc("fromsql"),
	Function("function"),
	FunctionType("function-type"),
	GenAlterCmds("gen-alter-cmds"),
	GenDisabledObjsCode("gen-disabled-objs-code"),
	General("general"),
	Generated("generated"),
	Global("global"),
	Grant("grant"),
	GrantOp("grant-op"),
	GridColor("grid-color"),
	GridPattern("grid-pattern"),
	GridSize("grid-size"),
	Group("group"),
	HandlerFunc("handler"),
	HasChanges("has-changes"),
	HasOptions("has-options"),
	Hashes("hashes"),
	HbaFile("hba-file"),
	Height("height"),
	HideCurPosZoomInfo("hide-cur-pos-zoom-info"),
	HideDescription("hide-description"),
	HideEmptyObjGroups("hide-empty-obj-groups"),
	HideExtAttribs("hide-ext-attribs"),
	HideObjectsSelInfo("hide-objects-sel-info"),
	HideObjShadows("hide-obj-shadows"),
	HideRelName("hide-rel-name"),
	HideSchNameUserTypes("hide-schema-names-of-types"),
	HideTableTags("hide-table-tags"),
	Highlight("highlight"),
	HighlightedText("highlighted-text"),
	HighlightLines("highlight-lines"),
	HighlightOrder("highlight-order"),
	HistoryMaxLength("history-max-length"),
	Icon("icon"),
	IconsSize("icons-size"),
	Id("id"),
	IdentFile("ident-file"),
	Identifier("identifier"),
	IdentityType("identity-type"),
	IgnoreDuplicErrors("ignore-duplic-errors"),
	IgnoreErrorCodes("ignore-error-codes"),
	IgnoreImportErrors("ignore-import-errors"),
	IgnoredChars("ignored-chars"),
	Implicit("implicit"),
	Import("import"),
	ImportExtObjs("import-ext-objs"),
	ImportSysObjs("import-sys-objs"),
	IncludedCols("include-cols"),
	Increment("increment"),
	Index("index"),
	IndexElement("idxelement"),
	IndexType("index-type"),
	Indexes("indexes"),
	Info("info"),
	InhColumn("inh-column"),
	InhColumns("inh-columns"),
	Inherit("inherit"),
	Inherited("inherited"),
	Inactive("inactive"),
	Initial("initial"),
	InitialCond("initial-cond"),
	InitialData("initial-data"),
	InitialExp("initial-exp"),
	InkSaver("inksaver"),
	InlineFunc("inline"),
	InputDatabase("input-db"),
	InputFunc("input"),
	InsEvent("ins-event"),
	InsertPriv("insert"),
	InternalLength("internal-length"),
	IntervalType("interval-type"),
	InvertRangeSelTrigger("invert-rangesel-trigger"),
	IoCast("io-cast"),
	IsExtType("is-ext-type"),
	IsPartitioned("is-partitioned"),
	IsTemplate("is-template"),
	Italic("italic"),
	Item("item"),
	Items("items"),
	JoinFunc("join"),
	KeepClusterObjs("keep-cluster-objs"),
	KeepObjsPerms("keep-objs-perms"),
	Label("label"),
	Labels("labels"),
	LabelsPos("labels-pos"),
	Landscape("landscape"),
	Language("language"),
	LastAnalyze("last-analyze"),
	LastAutovacuum("last-autovacuum"),
	LastPosition("last-position"),
	LastSysOid("last-sys-oid"),
	LastVacuum("last-vacuum"),
	LastValue("last-value"),
	LastZoom("last-zoom"),
	Layer("layer"),
	LayerNameColors("layer-name-colors"),
	LayerRectColors("layer-rect-colors"),
	Layers("layers"),
	LayersConfig("layersconfig"),
	LcCollate("lc-collate"),
	LcCollateMod("lc-collate-mod"),
	LcCtype("lc-ctype"),
	LcCtypeMod("lc-ctype-mod"),
	LeakProof("leakproof"),
	Left("left"),
	LeftType("left-type"),
	Length("length"),
	Library("library"),
	Light("light"),
	LikeType("like-type"),
	Line("line"),
	LineHighlightColor("line-highlight-color"),
	LineNumbersBgColor("line-numbers-bg-color"),
	LineNumbersColor("line-numbers-color"),
	Link("link"),
	LinkMode("link-mode"),
	LinkVisited("link-visited"),
	ListenAddresses("listen-addresses"),
	Locale("locale"),
	LocaleMod("locale-mod"),
	LockerArc("locker-arc"),
	LockerBody("locker-body"),
	LockPageDelimResize("lock-page-delim-resize"),
	Login("login"),
	LookaheadChar("lookahead-char"),
	LowVerbosity("low-verbosity"),
	MatchBySignature("match-by-signature"),
	Materialized("materialized"),
	MaxConnections("max-connections"),
	MaxObjCount("max-obj-count"),
	MaxValue("max-value"),
	Maximized("maximized"),
	Medium("medium"),
	Member("member"),
	MemberRoles("member-roles"),
	Merges("merges"),
	Metadata("metadata"),
	Mid("mid"),
	Midlight("midlight"),
	MinObjectOpacity("min-object-opacity"),
	MinValue("min-value"),
	Mode("mode"),
	Name("name"),
	NameColor("name-color"),
	NameFilter("name-filter"),
	NameLabel("name-label"),
	NamePatterns("name-patterns"),
	Names("names"),
	NegatorOp("negator-op"),
	NewIdentityType("new-identity-type"),
	NewName("new-name"),
	NewTableName("new-table-name"),
	NewVersion("new-version"),
	Next("next"),
	NnColumn("nn-column"),
	NoInherit("no-inherit"),
	None("none"),
	NotExtObject("not-ext-object"),
	NotNull("not-null"),
	NullsFirst("nulls-first"),
	NullsNotDistinct("nulls-not-distinct"),
	Object("object"),
	ObjectFinder("objectfinder"),
	ObjectId("object-id"),
	ObjectType("object-type"),
	Objects("objects"),
	ObjCount("objcount"),
	ObjRelationName("obj-rel-name"),
	ObjSelection("obj-selection"),
	ObjShadow("obj-shadow"),
	Oid("oid"),
	OidFilterOp("oid-filter-op"),
	Oids("oids"),
	OldName("old-name"),
	OldPgSqlVersions("old-pgsql-versions"),
	OldTableName("old-table-name"),
	OldVersion("old-version"),
	OnlyMatching("only-matching"),
	OpClass("opclass"),
	OpClasses("opclasses"),
	OpFamily("opfamily"),
	OpListSize("op-list-size"),
	OpenChar("open-char"),
	Operator("operator"),
	OperatorFunc("operfunc"),
	Operators("operators"),
	Options("options"),
	OriginalPk("original-pk"),
	OutputFunc("output"),
	Owner("owner"),
	OwnerColumn("owner-col"),
	Pagination("pagination"),
	PaperCustomSize("paper-custom-size"),
	PaperMargin("paper-margin"),
	PaperOrientation("paper-orientation"),
	PaperType("paper-type"),
	ParallelType("parallel-type"),
	ParamIn("in"),
	ParamOut("out"),
	ParamVariadic("variadic"),
	Parameter("parameter"),
	Parameters("parameters"),
	Parent("parent"),
	ParentType("parent-type"),
	Parents("parents"),
	Parsable("parsable"),
	PartialMatch("partial-match"),
	PartitionBoundExpr("partition-bound-expr"),
	PartitionKey("partitionkey"),
	PartitionTables("partition-tables"),
	PartitionedTable("partitioned-table"),
	Partitioning("partitioning"),
	PartKeyColls("part-key-colls"),
	PartKeyCols("part-key-cols"),
	PartKeyExprs("part-key-exprs"),
	PartKeyOpCls("part-key-opcls"),
	Password("password"),
	PasswordEncryption("password-encryption"),
	Path("path"),
	Patterns("patterns"),
	PerRow("per-line"),
	Permission("permission"),
	Permissive("permissive"),
	Persistent("persistent"),
	PgModelerVersion("pgmodeler-ver"),
	PgSqlBaseType("basetype"),
	PgSqlVersion("pgsql-ver"),
	PkColPattern("pk-col-pattern"),
	PkColumn("pk-column"),
	PkConstr("pk-constr"),
	PkPattern("pk-pattern"),
	Placeholder("placeholder"),
	PlaceholderText("placeholder-text"),
	Placeholders("placeholders"),
	Plugin("plugin"),
	Points("points"),
	Port("port"),
	Portrait("portrait"),
	Position("position"),
	PositionInfo("pos-info"),
	Precision("precision"),
	Predicate("predicate"),
	Preferred("preferred"),
	PrependAtBod("prepend-at-bod"),
	PrependSchema("prepend-schema"),
	PrependedSql("prepended-sql"),
	PreserveDbName("preserve-db-name"),
	Preset("preset"),
	Previous("previous"),
	PrintGrid("print-grid"),
	PrintPgNum("print-pg-num"),
	Privileges("privileges"),
	PrivilegesGop("privileges-gop"),
	ProtColumn("prot-column"),
	ProtItem("prot-item"),
	ProtItemAlt("prot-item-alt"),
	Protected("protected"),
	Provider("provider"),
	RangeAttribs("range-attribs"),
	RangeType("range"),
	Recent("recent"),
	RecentModels("recent-models"),
	RecreateUnmodObjs("recreate-unmod-objs"),
	Recursive("recursive"),
	RecvFunc("receive"),
	RectVisible("rect-visible"),
	ReducedForm("reduced-form"),
	RefAlias("ref-alias"),
	RefColumn("ref-column"),
	RefColumns("ref-columns"),
	RefName("ref-name"),
	RefTable("ref-table"),
	RefTableTag("reftable"),
	RefTables("ref-tables"),
	RefType("ref-type"),
	Refer("refer"),
	Reference("reference"),
	ReferenceFk("reference-fk"),
	References("references"),
	ReferencesPriv("references"),
	Referrers("referrers"),
	RegularExp("regexp"),
	RelAddedItem("rel-added-item"),
	RelAddedItemAlt("rel-added-item-alt"),
	RelatedForeignKey("related-foreign-key"),
	Relationship("relationship"),
	Relationship11("rel11"),
	Relationship1n("rel1n"),
	RelationshipDep("reldep"),
	RelationshipFk("relfk"),
	RelationshipGen("relgen"),
	RelationshipNn("relnn"),
	RelationshipPart("relpart"),
	RelationshipTabView("reltv"),
	RelsFollowTabsVisibility("rels-follow-tabs-visibility"),
	RemovedItem("removed-item"),
	Rename("rename"),
	ReplaceModObjs("replace-mod-objs"),
	Replication("replication"),
	RestartSeq("restart-seq"),
	RestrictionFunc("restriction"),
	ReturnTable("return-table"),
	ReturnType("return-type"),
	ReturnsSetOf("returns-setof"),
	ReuseSequences("reuse-sequences"),
	Revoke("revoke"),
	Right("right"),
	RightType("right-type"),
	RlsEnabled("rls-enabled"),
	RlsForced("rls-forced"),
	Role("role"),
	RoleMembers("rolemembers"),
	RoleType("role-type"),
	Roles("roles"),
	RowAmount("row-amount"),
	Rules("rules"),
	RunInTransaction("run-in-transaction"),
	SaveLastPosition("save-last-position"),
	SaveRestoreGeometry("save-restore-geometry"),
	SceneRect("scene-rect"),
	Schema("schema"),
	SchemaOid("schema-oid"),
	Schemas("schemas"),
	SearchPath("search-path"),
	SecurityBarrier("security-barrier"),
	SecurityInvoker("security-invoker"),
	SecurityType("security-type"),
	SelectExp("select-exp"),
	SelectObjects("select-objects"),
	SelectPriv("select"),
	SendFunc("send"),
	Sequence("sequence"),
	Sequences("sequences"),
	Server("server"),
	ServerEncoding("server-encoding"),
	ServerPid("server-pid"),
	ServerProtocol("server-protocol"),
	ServerVersion("server-version"),
	SessionOpts("sessionopts"),
	SetPerms("set-perms"),
	Shadow("shadow"),
	SharedObj("shared-obj"),
	ShellTypes("shell-types"),
	ShowAttributesGrid("show-attributes-grid"),
	ShowCanvasGrid("show-canvas-grid"),
	ShowLayerNames("show-layer-names"),
	ShowLayerRects("show-layer-rects"),
	ShowMainMenu("show-main-menu"),
	ShowPageDelimiters("show-page-delimiters"),
	ShowSourcePane("show-source-pane"),
	ShowSysSchemasRects("show-sys-schemas-rects"),
	Signature("signature"),
	SimpleCol("simplecol"),
	SimpleExp("simple-exp"),
	SimplifiedObjCreation("simplified-obj-creation"),
	SingleExp("single-exp"),
	SinglePkColumn("single-pk-col"),
	Size("size"),
	Small("small"),
	Snippet("snippet"),
	SortOp("sort-op"),
	SourceEditorApp("source-editor-app"),
	SourceEditorArgs("source-editor-args"),
	SourceType("source-type"),
	SpatialType("spatial-type"),
	SpecialPkCols("special-pk-cols"),
	Split("split"),
	SqlDisabled("sql-disabled"),
	SqlObject("sql-object"),
	SqlTool("sqltool"),
	SqlValidation("sql-validation"),
	Square("square"),
	SrcColPattern("src-col-pattern"),
	SrcColumns("src-columns"),
	SrcEncoding("src-encoding"),
	SrcFkPattern("src-fk-pattern"),
	SrcLabel("src-label"),
	SrcRequired("src-required"),
	SrcTable("src-table"),
	SrcType("src-type"),
	Srid("srid"),
	Ssl("ssl"),
	SslCaFile("ssl-ca-file"),
	SslCertFile("ssl-cert-file"),
	SslCrlFile("ssl-crl-file"),
	SslKeyFile("ssl-key-file"),
	StableBeta("stablebeta"),
	StableOnly("stableonly"),
	Start("start"),
	StateType("state-type"),
	Stikeout("strikeout"),
	Storage("storage"),
	StorageParams("stg-params"),
	StoreInFile("store-in-file"),
	StrategyNum("stg-number"),
	Style("style"),
	Styles("styles"),
	Subtype("subtype"),
	SubtypeDiffFunc("subtypediff"),
	Superuser("superuser"),
	Symbol("symbol"),
	System("system"),
	Table("table"),
	TableBody("table-body"),
	TableExtBody("table-ext-body"),
	TableName("table-name"),
	TableObject("table-obj"),
	TableSchemaName("table-schema-name"),
	TableTitle("table-title"),
	TableTogglerBody("table-toggler-body"),
	TableTogglerButtons("table-toggler-btns"),
	TableType("table-type"),
	Tablespace("tablespace"),
	TabWidth("tab-width"),
	Tag("tag"),
	TemplateDb("template"),
	TemporaryPriv("temporary"),
	Text("text"),
	ToolTipBase("tool-tip-base"),
	ToolTipText("tool-tip-text"),
	Top("top"),
	ToSqlFunc("tosql"),
	TpmodInFunc("tpmodin"),
	TpmodOutFunc("tpmodout"),
	TransformTypes("transform-types"),
	TransitionFunc("transition"),
	TriggerFunc("trigger-func"),
	TriggerPriv("trigger"),
	Triggers("triggers"),
	True("true"),
	Truncate("truncate"),
	TruncateColumnData("truncate-column-data"),
	TruncatePriv("truncate"),
	TruncEvent("trunc-event"),
	Trusted("trusted"),
	TuplesDel("tuples-del"),
	TuplesIns("tuples-ins"),
	Type("type"),
	TypeAttribute("typeattrib"),
	TypeClass("type-class"),
	TypeOid("type-oid"),
	Types("types"),
	TyplesUpd("tuples-upd"),
	UiLanguage("ui-language"),
	UiTheme("ui-theme"),
	Undefined("undefined"),
	Underline("underline"),
	Unique("unique"),
	Unlogged("unlogged"),
	Unset("unset"),
	UnsetPerms("unset-perms"),
	UpdAction("upd-action"),
	UpdEvent("upd-event"),
	Updated("updated"),
	UpdatedItem("updated-item"),
	UpdatePriv("update"),
	UqColumn("uq-column"),
	UqConstr("uq-constr"),
	UqPattern("uq-pattern"),
	UsagePriv("usage"),
	UseChangelog("use-changelog"),
	UseColumns("use-columns"),
	UseCurvedLines("use-curved-lines"),
	UseDefDisambiguation("use-def-disambiguation"),
	UsePlaceholders("use-placeholders"),
	UseSignature("use-signature"),
	UseSorting("use-sorting"),
	UseUniqueNames("use-unique-names"),
	UsingExp("using-exp"),
	Validation("validation"),
	Validator("validator"),
	ValidatorFunc("validator"),
	Validity("validity"),
	Value("value"),
	Values("values"),
	Variable("variable"),
	Variation("variation"),
	Version("version"),
	ViewBody("view-body"),
	ViewExtBody("view-ext-body"),
	ViewName("view-name"),
	ViewSchemaName("view-schema-name"),
	ViewTitle("view-title"),
	ViewTogglerBody("view-toggler-body"),
	ViewTogglerButtons("view-toggler-btns"),
	Visible("visible"),
	Widget("widget"),
	WidgetsGeometry("widgets-geometry"),
	Width("width"),
	Wildcard("wildcard"),
	Window("window"),
	WindowFunc("window-func"),
	WindowText("window-text"),
	WithNoData("with-no-data"),
	WithTimezone("with-timezone"),
	WithoutOids("without-oids"),
	Word("word"),
	WordDelimiters("word-delimiters"),
	WordSeparators("word-separators"),
	WorkingDir("working-dir"),
	XPos("x"),
	Year("year"),
	YPos("y"),
	ZValue("z-value");
}
