/*
# PostgreSQL Database Modeler (pgModeler)
#
# (c) Copyright 2006-2026 - Raphael Araújo e Silva <raphael@pgmodeler.io>
#
# DEVELOPMENT, MAINTENANCE AND COMMERCIAL DISTRIBUTION BY:
# Nullptr Labs Software e Tecnologia LTDA <contact@nullptrlabs.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 3.
#
# This program is distributed in the hope that it will be useful),
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# The complete text of GPLv3 is at LICENSE file on source code root directory.
# Also), you can get the complete GNU General Public License at <http://www.gnu.org/licenses/>
*/

/**
\ingroup libparsers
\namespace Attributes
\brief Definition of parsers attributes namespace which stores a series of static strings constants used
 to reference the attributes of objects in SQL/XML generation methods.
 Each string stores the name of the attribute used in the schema file "sch" of the respective objects.
\note <strong>Creation date:</strong> 23/09/2008
*/

#ifndef ATTRIBUTES_H
#define ATTRIBUTES_H

#include "parsersglobal.h"
#include <QString>

namespace Attributes {
extern __libparsers const QString
	Accent,
	Action,
	Active,
	ActiveLayers,
	AddedItem,
	AdminOption,
	AdminRoles,
	AlertApplyMetadata,
	AlertDisableQuoting,
	AlertOpenSqlTabs,
	AlertUnsavedModels,
	Alias,
	Alignment,
	AlignObjsToGrid,
	AllowCompletion,
	AllowConns,
	AllVersions,
	AlternateBase,
	AlterCmds,
	AnalyzeFunc,
	AncestorTable,
	Any,
	AppendAtEod,
	AppendedSql,
	Application,
	ApplyOnServer,
	ArgCount,
	ArgDefCount,
	ArgDefaults,
	ArgModes,
	ArgNames,
	ArgTypes,
	Arguments,
	AscOrder,
	Assignment,
	Attribute,
	Attributes,
	AttribsPage,
	AttribsPerPage,
	AutoBrowseDb,
	AutoSaveInterval,
	Author,
	BgColor,
	Base,
	BaseType,
	Before,
	BehaviorType,
	Big,
	BlogPost,
	Bold,
	BorderColor,
	Bottom,
	BrightText,
	Buffering,
	Button,
	ButtonText,
	BypassRls,
	ByValue,
	Cache,
	CanonicalFunc,
	CanvasColor,
	CanvasCornerMove,
	CaptureNearby,
	Cascade,
	CaseSensitive,
	CastType,
	Category,
	Change,
	Changelog,
	CheckExp,
	CheckOption,
	CheckUpdate,
	CheckVersions,
	CkConstr,
	ClientEncoding,
	CloseChar,
	Code,
	CodeCompletion,
	ColIndexes,
	ColIsIdentity,
	CollapseMode,
	Collatable,
	Collation,
	Collations,
	Color,
	Colors,
	ColsComment,
	Column,
	ColumnAlias,
	ColumnTruncThreshold,
	Columns,
	Command,
	Commands,
	Comment,
	CommutatorOp,
	CompactView,
	CompareToDatabase,
	ComparisonType,
	CompletionTrigger,
	CompositeType,
	Concurrent,
	Condition,
	ConfigFile,
	ConfigParam,
	ConfigParams,
	Configuration,
	ConfirmValidation,
	ConnectCenterPnts,
	ConnectFkToPk,
	Connection,
	ConnectionTimeout,
	Connections,
	ConnectPriv,
	ConnectTableEdges,
	ConnLimit,
	Constraint,
	Constraints,
	ConstrDefs,
	ConstrIndexes,
	ConstrSqlDisabled,
	Contents,
	CopyMode,
	CopyOptions,
	CopyTable,
	Create,
	CreateCmds,
	CreateDb,
	CreatePriv,
	CreateRole,
	Created,
	CrowsFoot,
	CteExpression,
	CurIdentityType,
	CurVersion,
	CurrentModel,
	CustomColor,
	CustomFilter,
	CustomIdxs,
	CustomScale,
	Cycle,
	Dark,
	DataDirectory,
	DataDictIndex,
	Database,
	Date,
	DbModel,
	DdlEndToken,
	DeadRowsAmount,
	DebugMode,
	Declaration,
	DeclInTable,
	Default,
	DefaultCollation,
	DefaultForOps,
	DefaultOwner,
	DefaultSchema,
	DefaultTablespace,
	DefaultValue,
	Deferrable,
	DeferType,
	Definition,
	DelAction,
	DelEvent,
	Deleted,
	DeletePriv,
	Delimiter,
	DelimitersColor,
	Description,
	Design,
	DestType,
	Deterministic,
	Diff,
	DiffDatabases,
	DiffModelDb,
	DiffModels,
	DifMode,
	Dimension,
	Directory,
	Disabled,
	DisableInlineEditor,
	DisableNameQuoting,
	DisableSmoothness,
	DisplayLineNumbers,
	DockWidgets,
	DomConstraint,
	DontDropMissingObjs,
	Dot,
	Drop,
	DropCmds,
	DropMissingColsConstrs,
	DropTruncCascade,
	DstColPattern,
	DstColumns,
	DstEncoding,
	DstFkPattern,
	DstLabel,
	DstRequired,
	DstTable,
	DstType,
	DynamicLibraryPath,
	DynamicSharedMemory,
	Element,
	Elements,
	ElementsCount,
	EmptyPassword,
	Encoding,
	EnclosingChars,
	Encrypted,
	EndExp,
	EntireLine,
	Entry,
	EnumType,
	EscapeComment,
	Event,
	EventType,
	Events,
	ExactMatch,
	ExcBuiltinArrays,
	ExcludeElement,
	ExConstr,
	ExecType,
	ExecutPriv,
	ExecutionCost,
	ExistingValue,
	ExpansionFactor,
	Explicit,
	Export,
	ExportToFile,
	Expression,
	Expressions,
	ExtAttribsPage,
	ExtAttribsPerPage,
	ExtObjOids,
	ExtraCondition,
	ExtraInfo,
	Factor,
	FadedOut,
	FadeInObjects,
	False,
	Family,
	FastUpdate,
	Fdw,
	File,
	FileAssociated,
	FillColor,
	Filter,
	FilterOids,
	FilterTableTypes,
	Filters,
	Final,
	FinalExp,
	FinalFunc,
	FiringType,
	FirstRun,
	FkColumn,
	FkConstr,
	FkDefs,
	FkIdxPattern,
	FkIdxType,
	Font,
	FontSize,
	ForceObjsReCreation,
	ForcedFiltering,
	ForeignKeys,
	ForeignTableBody,
	ForeignTableExtBody,
	ForeignTableName,
	ForeignTableSchemaName,
	ForeignTableTitle,
	ForeignTableTogglerBody,
	ForeignTableTogglerButtons,
	FgColor,
	FormatName,
	FromExp,
	FromSqlFunc,
	Function,
	FunctionType,
	GenAlterCmds,
	GenDisabledObjsCode,
	General,
	Generated,
	Global,
	Grant,
	GrantOp,
	GridColor,
	GridPattern,
	GridSize,
	Group,
	HandlerFunc,
	HasChanges,
	HasOptions,
	Hashes,
	HbaFile,
	Height,
	HideCurPosZoomInfo,
	HideDescription,
	HideEmptyObjGroups,
	HideExtAttribs,
	HideObjectsSelInfo,
	HideObjShadows,
	HideRelName,
	HideSchNameUserTypes,
	HideTableTags,
	Highlight,
	HighlightedText,
	HighlightLines,
	HighlightOrder,
	HistoryMaxLength,
	Icon,
	IconsSize,
	Id,
	IdentFile,
	Identifier,
	IdentityType,
	IgnoreDuplicErrors,
	IgnoreErrorCodes,
	IgnoreImportErrors,
	IgnoredChars,
	Implicit,
	Import,
	ImportExtObjs,
	ImportSysObjs,
	IncludedCols,
	Increment,
	Index,
	IndexElement,
	IndexType,
	Indexes,
	Info,
	InhColumn,
	InhColumns,
	Inherit,
	Inherited,
	Inactive,
	Initial,
	InitialCond,
	InitialData,
	InitialExp,
	InkSaver,
	InlineFunc,
	InputDatabase,
	InputFunc,
	InsEvent,
	InsertPriv,
	InternalLength,
	IntervalType,
	InvertRangeSelTrigger,
	IoCast,
	IsExtType,
	IsPartitioned,
	IsTemplate,
	Italic,
	Item,
	Items,
	JoinFunc,
	KeepClusterObjs,
	KeepObjsPerms,
	Label,
	Labels,
	LabelsPos,
	Landscape,
	Language,
	LastAnalyze,
	LastAutovacuum,
	LastPosition,
	LastSysOid,
	LastVacuum,
	LastValue,
	LastZoom,
	Layer,
	LayerNameColors,
	LayerRectColors,
	Layers,
	LayersConfig,
	LcCollate,
	LcCollateMod,
	LcCtype,
	LcCtypeMod,
	LeakProof,
	Left,
	LeftType,
	Length,
	Library,
	Light,
	LikeType,
	Line,
	LineHighlightColor,
	LineNumbersBgColor,
	LineNumbersColor,
	Link,
	LinkMode,
	LinkVisited,
	ListenAddresses,
	Locale,
	LocaleMod,
	LockerArc,
	LockerBody,
	LockPageDelimResize,
	Login,
	LookaheadChar,
	LowVerbosity,
	MatchBySignature,
	Materialized,
	MaxConnections,
	MaxObjCount,
	MaxValue,
	Maximized,
	Medium,
	Member,
	MemberRoles,
	Merges,
	Metadata,
	Mid,
	Midlight,
	MinObjectOpacity,
	MinValue,
	Mode,
	Name,
	NameColor,
	NameFilter,
	NameLabel,
	NamePatterns,
	Names,
	NegatorOp,
	NewIdentityType,
	NewName,
	NewTableName,
	NewVersion,
	Next,
	NnColumn,
	NoInherit,
	None,
	NotExtObject,
	NotNull,
	NullsFirst,
	NullsNotDistinct,
	Object,
	ObjectFinder,
	ObjectId,
	ObjectType,
	Objects,
	ObjCount,
	ObjRelationName,
	ObjSelection,
	ObjShadow,
	Oid,
	OidFilterOp,
	Oids,
	OldName,
	OldPgSqlVersions,
	OldTableName,
	OldVersion,
	OnlyMatching,
	OpClass,
	OpClasses,
	OpFamily,
	OpListSize,
	OpenChar,
	Operator,
	OperatorFunc,
	Operators,
	Options,
	OriginalPk,
	OutputFunc,
	Owner,
	OwnerColumn,
	Pagination,
	PaperCustomSize,
	PaperMargin,
	PaperOrientation,
	PaperType,
	ParallelType,
	ParamIn,
	ParamOut,
	ParamVariadic,
	Parameter,
	Parameters,
	Parent,
	ParentType,
	Parents,
	Parsable,
	PartialMatch,
	PartitionBoundExpr,
	PartitionKey,
	PartitionTables,
	PartitionedTable,
	Partitioning,
	PartKeyColls,
	PartKeyCols,
	PartKeyExprs,
	PartKeyOpCls,
	Password,
	PasswordEncryption,
	Path,
	Patterns,
	PerRow,
	Permission,
	Permissive,
	Persistent,
	PgModelerVersion,
	PgSqlBaseType,
	PgSqlVersion,
	PkColPattern,
	PkColumn,
	PkConstr,
	PkPattern,
	Placeholder,
	PlaceholderText,
	Placeholders,
	Plugin,
	Points,
	Port,
	Portrait,
	Position,
	PositionInfo,
	Precision,
	Predicate,
	Preferred,
	PrependAtBod,
	PrependSchema,
	PrependedSql,
	PreserveDbName,
	Preset,
	Previous,
	PrintGrid,
	PrintPgNum,
	Privileges,
	PrivilegesGop,
	ProtColumn,
	ProtItem,
	ProtItemAlt,
	Protected,
	Provider,
	RangeAttribs,
	RangeType,
	Recent,
	RecentModels,
	RecreateUnmodObjs,
	Recursive,
	RecvFunc,
	RectVisible,
	ReducedForm,
	RefAlias,
	RefColumn,
	RefColumns,
	RefName,
	RefTable,
	RefTableTag,
	RefTables,
	RefType,
	Refer,
	Reference,
	ReferenceFk,
	References,
	ReferencesPriv,
	Referrers,
	RegularExp,
	RelAddedItem,
	RelAddedItemAlt,
	RelatedForeignKey,
	Relationship,
	Relationship11,
	Relationship1n,
	RelationshipDep,
	RelationshipFk,
	RelationshipGen,
	RelationshipNn,
	RelationshipPart,
	RelationshipTabView,
	RelsFollowTabsVisibility,
	RemovedItem,
	Rename,
	ReplaceModObjs,
	Replication,
	RestartSeq,
	RestrictionFunc,
	ReturnTable,
	ReturnType,
	ReturnsSetOf,
	ReuseSequences,
	Revoke,
	Right,
	RightType,
	RlsEnabled,
	RlsForced,
	Role,
	RoleMembers,
	RoleType,
	Roles,
	RowAmount,
	Rules,
	RunInTransaction,
	SaveLastPosition,
	SaveRestoreGeometry,
	SceneRect,
	Schema,
	SchemaOid,
	Schemas,
	SearchPath,
	SecurityBarrier,
	SecurityInvoker,
	SecurityType,
	SelectExp,
	SelectObjects,
	SelectPriv,
	SendFunc,
	Sequence,
	Sequences,
	Server,
	ServerEncoding,
	ServerPid,
	ServerProtocol,
	ServerVersion,
	SessionOpts,
	SetPerms,
	Shadow,
	SharedObj,
	ShellTypes,
	ShowAttributesGrid,
	ShowCanvasGrid,
	ShowLayerNames,
	ShowLayerRects,
	ShowMainMenu,
	ShowPageDelimiters,
	ShowSourcePane,
	ShowSysSchemasRects,
	Signature,
	SimpleCol,
	SimpleExp,
	SimplifiedObjCreation,
	SingleExp,
	SinglePkColumn,
	Size,
	Small,
	Snippet,
	SortOp,
	SourceEditorApp,
	SourceEditorArgs,
	SourceType,
	SpatialType,
	SpecialPkCols,
	Split,
	SqlDisabled,
	SqlObject,
	SqlTool,
	SqlValidation,
	Square,
	SrcColPattern,
	SrcColumns,
	SrcEncoding,
	SrcFkPattern,
	SrcLabel,
	SrcRequired,
	SrcTable,
	SrcType,
	Srid,
	Ssl,
	SslCaFile,
	SslCertFile,
	SslCrlFile,
	SslKeyFile,
	StableBeta,
	StableOnly,
	Start,
	StateType,
	Stikeout,
	Storage,
	StorageParams,
	StoreInFile,
	StrategyNum,
	Style,
	Styles,
	Subtype,
	SubtypeDiffFunc,
	Superuser,
	Symbol,
	System,
	Table,
	TableBody,
	TableExtBody,
	TableName,
	TableObject,
	TableSchemaName,
	TableTitle,
	TableTogglerBody,
	TableTogglerButtons,
	TableType,
	Tablespace,
	TabWidth,
	Tag,
	TemplateDb,
	TemporaryPriv,
	Text,
	ToolTipBase,
	ToolTipText,
	Top,
	ToSqlFunc,
	TpmodInFunc,
	TpmodOutFunc,
	TransformTypes,
	TransitionFunc,
	TriggerFunc,
	TriggerPriv,
	Triggers,
	True,
	Truncate,
	TruncateColumnData,
	TruncatePriv,
	TruncEvent,
	Trusted,
	TuplesDel,
	TuplesIns,
	Type,
	TypeAttribute,
	TypeClass,
	TypeOid,
	Types,
	TyplesUpd,
	UiLanguage,
	UiTheme,
	Undefined,
	Underline,
	Unique,
	Unlogged,
	Unset,
	UnsetPerms,
	UpdAction,
	UpdEvent,
	Updated,
	UpdatedItem,
	UpdatePriv,
	UqColumn,
	UqConstr,
	UqPattern,
	UsagePriv,
	UseChangelog,
	UseColumns,
	UseCurvedLines,
	UseDefDisambiguation,
	UsePlaceholders,
	UseSignature,
	UseSorting,
	UseUniqueNames,
	UsingExp,
	Validation,
	Validator,
	ValidatorFunc,
	Validity,
	Value,
	Values,
	Variable,
	Variation,
	Version,
	ViewBody,
	ViewExtBody,
	ViewName,
	ViewSchemaName,
	ViewTitle,
	ViewTogglerBody,
	ViewTogglerButtons,
	Visible,
	Widget,
	WidgetsGeometry,
	Width,
	Wildcard,
	Window,
	WindowFunc,
	WindowText,
	WithNoData,
	WithTimezone,
	WithoutOids,
	Word,
	WordDelimiters,
	WordSeparators,
	WorkingDir,
	XPos,
	Year,
	YPos,
	ZValue;
}

#endif
